@@ -26,11 +26,12 @@ import testscenarios
 
 from oslo.config import cfg
 from oslo import messaging
-from oslo.messaging._drivers import amqpdriver
-from oslo.messaging._drivers import common as driver_common
-from oslo.messaging._drivers import impl_rabbit as rabbit_driver
 from oslo.serialization import jsonutils
-from tests import utils as test_utils
+from oslo_messaging._drivers import amqp
+from oslo_messaging._drivers import amqpdriver
+from oslo_messaging._drivers import common as driver_common
+from oslo_messaging._drivers import impl_rabbit as rabbit_driver
+from oslo_messaging.tests import utils as test_utils
 
 load_tests = testscenarios.load_tests_apply_scenarios
 
@@ -41,9 +42,11 @@ class TestDeprecatedRabbitDriverLoad(test_utils.BaseTestCase):
         super(TestDeprecatedRabbitDriverLoad, self).setUp(
             conf=cfg.ConfigOpts())
         self.messaging_conf.transport_driver = 'rabbit'
-        self.config(fake_rabbit=True)
+        self.config(fake_rabbit=True, group="oslo_messaging_rabbit")
 
     def test_driver_load(self):
+        self.config(heartbeat_timeout_threshold=0,
+                    group='oslo_messaging_rabbit')
         transport = messaging.get_transport(self.conf)
         self.addCleanup(transport.cleanup)
         driver = transport._driver
@@ -55,16 +58,28 @@ class TestDeprecatedRabbitDriverLoad(test_utils.BaseTestCase):
 
 class TestRabbitDriverLoad(test_utils.BaseTestCase):
 
-    def setUp(self):
-        super(TestRabbitDriverLoad, self).setUp()
-        self.messaging_conf.transport_driver = 'rabbit'
+    scenarios = [
+        ('rabbit', dict(transport_driver='rabbit',
+                        url='amqp://guest:guest@localhost:5672//')),
+        ('kombu', dict(transport_driver='kombu',
+                       url='amqp://guest:guest@localhost:5672//')),
+        ('rabbit+memory', dict(transport_driver='kombu+memory',
+                               url='memory:///'))
+    ]
 
-    @mock.patch('oslo.messaging._drivers.impl_rabbit.Connection.ensure')
-    @mock.patch('oslo.messaging._drivers.impl_rabbit.Connection.reset')
+    @mock.patch('oslo_messaging._drivers.impl_rabbit.Connection.ensure')
+    @mock.patch('oslo_messaging._drivers.impl_rabbit.Connection.reset')
     def test_driver_load(self, fake_ensure, fake_reset):
+        self.config(heartbeat_timeout_threshold=0,
+                    group='oslo_messaging_rabbit')
+        self.messaging_conf.transport_driver = self.transport_driver
         transport = messaging.get_transport(self.conf)
         self.addCleanup(transport.cleanup)
-        self.assertIsInstance(transport._driver, rabbit_driver.RabbitDriver)
+        driver = transport._driver
+        url = driver._get_connection()._url
+
+        self.assertIsInstance(driver, rabbit_driver.RabbitDriver)
+        self.assertEqual(self.url, url)
 
 
 class TestRabbitIterconsume(test_utils.BaseTestCase):
@@ -73,7 +88,7 @@ class TestRabbitIterconsume(test_utils.BaseTestCase):
         transport = messaging.get_transport(self.conf, 'kombu+memory:////')
         self.addCleanup(transport.cleanup)
         deadline = time.time() + 3
-        with transport._driver._get_connection() as conn:
+        with transport._driver._get_connection(amqp.PURPOSE_LISTEN) as conn:
             conn.iterconsume(timeout=3)
             # kombu memory transport doesn't really raise error
             # so just simulate a real driver behavior
@@ -120,20 +135,34 @@ class TestRabbitTransportURL(test_utils.BaseTestCase):
               expected=["amqp://user:password@host:10/virtual_host",
                         "amqp://user2:password2@host2:12/virtual_host"]
               )),
+        ('qpid',
+         dict(url='kombu+qpid://user:password@host:10/virtual_host',
+              expected=['qpid://user:password@host:10/virtual_host'])),
+        ('rabbit',
+         dict(url='kombu+rabbit://user:password@host:10/virtual_host',
+              expected=['amqp://user:password@host:10/virtual_host'])),
     ]
 
     def setUp(self):
         super(TestRabbitTransportURL, self).setUp()
+        self.config(heartbeat_timeout_threshold=0,
+                    group='oslo_messaging_rabbit')
         self.messaging_conf.transport_driver = 'rabbit'
 
-    @mock.patch('oslo.messaging._drivers.impl_rabbit.Connection.ensure')
-    @mock.patch('oslo.messaging._drivers.impl_rabbit.Connection.reset')
+    @mock.patch('oslo_messaging._drivers.impl_rabbit.Connection.ensure')
+    @mock.patch('oslo_messaging._drivers.impl_rabbit.Connection.reset')
     def test_transport_url(self, fake_ensure_connection, fake_reset):
         transport = messaging.get_transport(self.conf, self.url)
         self.addCleanup(transport.cleanup)
         driver = transport._driver
 
-        urls = driver._get_connection()._url.split(";")
+        # NOTE(sileht): some kombu transport can depend on library that
+        # we don't want to depend yet, because selecting the transport
+        # is experimental, only amqp is supported
+        # for example kombu+qpid depends of qpid-tools
+        # so, mock the connection.info to skip call to qpid-tools
+        with mock.patch('kombu.connection.Connection.info'):
+            urls = driver._get_connection()._url.split(";")
         self.assertEqual(sorted(self.expected), sorted(urls))
 
 
@@ -178,6 +207,8 @@ class TestSendReceive(test_utils.BaseTestCase):
                                                          cls._timeout)
 
     def test_send_receive(self):
+        self.config(heartbeat_timeout_threshold=0,
+                    group='oslo_messaging_rabbit')
         transport = messaging.get_transport(self.conf, 'kombu+memory:////')
         self.addCleanup(transport.cleanup)
 
@@ -237,8 +268,27 @@ class TestSendReceive(test_utils.BaseTestCase):
                         raise ZeroDivisionError
                     except Exception:
                         failure = sys.exc_info()
-                    msgs[i].reply(failure=failure,
-                                  log_failure=not self.expected)
+
+                    # NOTE(noelbk) confirm that Publisher exchanges
+                    # are always declared with passive=True
+                    outer_self = self
+                    test_exchange_was_called = [False]
+                    old_init = kombu.entity.Exchange.__init__
+
+                    def new_init(self, *args, **kwargs):
+                        test_exchange_was_called[0] = True
+                        outer_self.assertTrue(kwargs['passive'])
+                        old_init(self, *args, **kwargs)
+                    kombu.entity.Exchange.__init__ = new_init
+
+                    try:
+                        msgs[i].reply(failure=failure,
+                                      log_failure=not self.expected)
+                    finally:
+                        kombu.entity.Exchange.__init__ = old_init
+
+                    self.assertTrue(test_exchange_was_called[0])
+
                 elif self.rx_id:
                     msgs[i].reply({'rx_id': i})
                 else:
@@ -657,7 +707,8 @@ class RpcKombuHATestCase(test_utils.BaseTestCase):
         self.config(rabbit_hosts=self.brokers,
                     rabbit_retry_interval=0.01,
                     rabbit_retry_backoff=0.01,
-                    kombu_reconnect_delay=0)
+                    kombu_reconnect_delay=0,
+                    group="oslo_messaging_rabbit")
 
         self.kombu_connect = mock.Mock()
         self.useFixture(mockpatch.Patch(
@@ -668,13 +719,14 @@ class RpcKombuHATestCase(test_utils.BaseTestCase):
 
         # starting from the first broker in the list
         url = messaging.TransportURL.parse(self.conf, None)
-        self.connection = rabbit_driver.Connection(self.conf, url)
+        self.connection = rabbit_driver.Connection(self.conf, url,
+                                                   amqp.PURPOSE_SEND)
         self.addCleanup(self.connection.close)
 
     def test_ensure_four_retry(self):
         mock_callback = mock.Mock(side_effect=IOError)
         self.assertRaises(messaging.MessageDeliveryFailure,
-                          self.connection.ensure, None, mock_callback,
+                          self.connection.ensure, mock_callback,
                           retry=4)
         self.assertEqual(5, self.kombu_connect.call_count)
         self.assertEqual(6, mock_callback.call_count)
@@ -682,7 +734,7 @@ class RpcKombuHATestCase(test_utils.BaseTestCase):
     def test_ensure_one_retry(self):
         mock_callback = mock.Mock(side_effect=IOError)
         self.assertRaises(messaging.MessageDeliveryFailure,
-                          self.connection.ensure, None, mock_callback,
+                          self.connection.ensure, mock_callback,
                           retry=1)
         self.assertEqual(2, self.kombu_connect.call_count)
         self.assertEqual(3, mock_callback.call_count)
@@ -690,7 +742,7 @@ class RpcKombuHATestCase(test_utils.BaseTestCase):
     def test_ensure_no_retry(self):
         mock_callback = mock.Mock(side_effect=IOError)
         self.assertRaises(messaging.MessageDeliveryFailure,
-                          self.connection.ensure, None, mock_callback,
+                          self.connection.ensure, mock_callback,
                           retry=0)
         self.assertEqual(1, self.kombu_connect.call_count)
         self.assertEqual(2, mock_callback.call_count)
